### Script to plot an irradiated disk spectrum for a static disk

import numpy as np
import const
from pylab import *
from scipy import *
import scipy.integrate


####Define all the default paramters

r_in = 3        ## ISCO radius in schwarzschild radii
r_out = 1e4     ## desired outer radius of accretion disk 
delta_r = 1     ## desired radial resolution 
M = 1e8         ## Black hole mass (solar masses)
D = 1 			## redshift distance to black hole
inc = 0         ## desired inclination (radians)
M_dot = 1 		## Accretion rate (solar masses per year)
beta = 0

## Convert to SI units
M = M*const.Msun
rs = const.r_sch(M)
r_in = r_in*rs
r_out = r_out*rs
delta_r = delta_r*rs
M_dot=M_dot * const.Msun





##define the temperature radius profile for a steady state accretion disk in parsecs
##viscous dissipation due gives rise to a factor of 3/8pi
def T_R(M,M_dot,R,R_in):
	T=(3*const.G*M*M_dot*(1-(R_in/R)**0.5)/(8*np.pi*const.sb*R**3))**0.25
	return(T)



### Luminosity of the central source (powered by acrretion. May need to introduce efficiency
##parameter later
Lum = const.G * M * M_dot / r_in

##populate the x and y dimensions of the array with radial and azimuthal coordinates 
rad_inf=1.*np.arange(np.ceil((r_out-r_in)/delta_r))/np.ceil((r_out-r_in)/delta_r)*(r_out-r_in) + r_in


	

### Obtain the temperature radius relation, wavelengths of the corresponding peak wavelengths
## and solid angle element of each radii
s_a=2*np.pi*rad_inf*delta_r*np.cos(inc)/D**2
T=T_R(M,M_dot,rad_inf,r_in)	
wavelength = const.w_c/T


##plank spectrum as a function of wavelength have to sum over temperature contributions from all of disk
#B_T=np.zeros((len(rad_inf),len(wavelength)))
#B_T=np.zeros(10)
#B_T_2=np.zeros(10)

#sum_ar=np.arange(len(rad_inf))
#sum_ar=np.arange(10)
#for i in range(10):
#	B_T[i]=(2*const.plank*const.c**2/wavelength[i]**3 * (np.exp(const.plank*const.c/(wavelength[i]*const.k*T)) - 1)**-1 * s_a).sum()

#	if i%100 != 0:
#		print ' done '+str(i)+ ' of ' + str(len(rad_inf))


#test_ar=np.zeros((len(rad_inf),len(T),10))
#test_ar[:
#B_T_2[:]=
#(2*const.plank*const.c**2/wavelength[sum_ar[:]]**3

#B_T_2[sum_ar[:]]=(2*const.plank*const.c**2/wavelength[sum_ar[:]]**3 * (np.exp(const.plank*const.c/(wavelength[sum_ar[:]]*const.k*T)) - 1)**-1 * s_a).sum()

#x=np.arange(len(rad_inf))
#y=np.arange(len(T))

#for z in range(len(wavelength)):
def plank(R,wav):
	#wav=1e-7
	flux=2*const.plank*const.c/wav**3 * (np.exp(const.plank*const.c/(wav*const.k*(3*const.G*M*M_dot*(1-(r_in/R)**0.5)/(8*np.pi*const.sb*R**3))**0.25)) - 1)**-1 * 2*np.pi*R*np.cos(inc)/D**2
	return(flux)

wav=1e-7


flux= lambda R: 2*const.plank*const.c/wavelength[:]**3 * (np.exp(const.plank*const.c/(wavelength[:]*const.k*(3*const.G*M*M_dot*(1-(r_in/R)**0.5)/(8*np.pi*const.sb*R**3))**0.25)) - 1)**-1 * 2*np.pi*R*np.cos(inc)/D**2
#output=scipy.integrate.quad(flux,rad_inf[1],rad_inf[-1])
	
	#d=np.zeros(10)
#d[z]=
#show()
wav=1e-7

def plank(R,wav):
	test=(2*const.plank*const.c**2/wav**3 * (np.exp(const.plank*const.c/(wav*const.k*(3*const.G*M*M_dot*(1-(r_in/rad_inf[:])**0.5)/(8*np.pi*const.sb*rad_inf[:]**3))**0.25)) - 1)**-1 * 2*np.pi*rad_inf[:]*delta_r*np.cos(inc)/D**2).sum()
	return(flux)

result=plank(rad_inf[:],wavelength[:])



	
### Define BTR

B_T_r = np.zeros((len(rad_inf), len(wavelength)))	
	
####Now define the emission associated by irradiation from point source
## A thick disc has a depth (H) as a function of radius (x) given by 
alpha=9./7
	
h_inf=rad_inf**9./7
diff_h=9./7*rad_inf**2/7
	
	
##solid angle element
##see note book for derivation
sa2=2*np.pi*rad_inf/D**2*(np.sin(inc)-np.cos(inc)**diff_h)
	
##Also different temperature profile (see note book or frank, king and rainne chapt 5.10)


T_r=(Lum/(4*np.pi*rad_inf**2*const.sb)*h_inf/rad_inf*(1.*alpha-1.)*(1-beta))**0.25

## flux. As with plank spectrum
#for i in range(len(rad_inf)):
#	if i%100 != 0:
#		print ' done '+str(i)+ ' of ' + str(len(rad_inf))
#	B_T_r[i]=(2*const.plank*const.c**2/wavelength[i]**3 * (np.exp(const.plank*const.c/(wavelength[i]*const.k*T_r)) - 1)**-1 * sa2).sum()
	
	
	


import numpy as np



def rw(ti,yi,si=0,tgi = -1,fbreak=-1,custom_freqs=[]):

 print(tgi[1:10])
 print(custom_freqs[:10])
 #initialise starting inputs and normalise
 twopi = 2*np.pi
 t = np.array(ti)
 tlo = np.min(t)
 thi = np.max(t)
 dt  = t[1] - t[0]
 ymean = np.mean(yi)
 ystd  = np.std(yi)
 y = (yi - ymean)/ystd
 ny = len(y)
 rms = np.std(y)
 if (type(si) == np.ndarray):
  s = si/ystd
 else:
  s = np.ones(ny)
 if (type(tgi) == np.ndarray): 
  tgrid = np.array(tgi)
 else:
  tgrid = np.arange(tlo,thi+dt,dt)
 ntgrid = len(tgrid)
 dtm = tgrid[1]-tgrid[0]
 tglo,tghi = tgrid[0],tgrid[-1]
 
 #initialise frequencies  
 fnyq = 0.5/dtm
 flo = 1.0/(tghi - tglo)
 fhi = fnyq 
 if (fbreak == -1):
  w0  = min(10*flo,fhi/2)*twopi
 else:
  w0 = twopi*fbreak
  
 if (custom_freqs == []): 
  f = np.arange(flo,fhi+flo,flo)
 else:
  f = custom_freqs
 
 w =f*twopi
 nw = np.shape(w)[0]
 np2 = 2*nw
 #define power spectrum prior
 w2 = (w/w0)**(-2)
 w2_integrate = w0**2 * (1/w[0] - 1/w[-1])
 #w2 = 1./(1+(w/w0)**2)
 #w2_integrate = w0*(1./np.tan(w[-1]/w0) - 1./np.tan(w[0]/w0))

 
 #bplin=[fbreak,2,2]
 #w2 = (1. + (w/(twopi*bplin[0]))**(bplin[1]-bplin[2])) / (w/(twopi*bplin[0]))**bplin[1]
 #
 #w_hi = w[-1]
 #w_lo = w[0]
 #w2_integrate_hi = w_hi*( (w_hi/(twopi*bplin[0]))**(-bplin[1])/(1.-bplin[1]) + (w_hi/(twopi*bplin[0]))**(-bplin[2])/(1.-bplin[2]) )
 #w2_integrate_lo = w_lo*( (w_lo/(twopi*bplin[0]))**(-bplin[1])/(1.-bplin[1]) + (w_lo/(twopi*bplin[0]))**(-bplin[2])/(1.-bplin[2]) )
 #w2_integrate = w2_integrate_hi - w2_integrate_lo
 
 
 
 
 
 
 
 #save the sine and cosine value in  2d array to speed computation
 cwt = np.ones((ny,nw))
 swt = np.ones((ny,nw))
 for iw in range(nw):
  cwt[:,iw] = np.cos(w[iw]*t)
  swt[:,iw] = np.sin(w[iw]*t)
 
 
 #perform hessian fit
 cvec = np.ones(np2)
 hes = np.ones((np2,np2))
 s2 = s*s
 cwtT = cwt.T
 swtT = swt.T
 
 hnow_cc = np.tensordot(cwtT/s2,cwt,axes=1)
 hnow_sc = np.tensordot(swtT/s2,cwt,axes=1)
 hnow_cs = np.tensordot(cwtT/s2,swt,axes=1)
 hnow_ss = np.tensordot(swtT/s2,swt,axes=1)

  
  
 import time
 t1 = time.time()
 idk = np.arange(0,np2,2)
 idk1 = np.arange(1,np2,2)
 hes[::2,::2] = hnow_cc[:,:].T
 hes[::2,1::2] = hnow_sc[:,:].T
 hes[1::2,::2] = hnow_cs[:,:].T
 hes[1::2,1::2] = hnow_ss[:,:].T 
 ##add on prior
 x = np.sqrt(w2_integrate)/(2*rms*w2)

 
 
 
 idk = np.arange(0,np2,2)
 idk1 = np.arange(1,np2,2)
 hes[::2,::2] = hnow_cc[:,:].T
 hes[::2,1::2] = hnow_sc[:,:].T
 hes[1::2,::2] = hnow_cs[:,:].T
 hes[1::2,1::2] = hnow_ss[:,:].T 
 
 ##add on prior unless fitting custom frequencies 
 if (custom_freqs == []):
  x = np.sqrt(w2_integrate)/(2*rms*w2)
 else:
  x = 0
  
 hes[idk,idk] = hes[idk,idk] + x
 hes[idk1,idk1] = hes[idk1,idk1] + x
 ys2 = y/s2
 cvec[idk] = np.dot(cwtT,ys2)
 cvec[idk1] = np.dot(swtT,ys2)

 t2 = time.time()
 
 print('time1',t2-t1)

 
 #t1 = time.time()
 #iwp = 0
 #for ikp in range(0,np2,2):
 # wp = w[iwp]
 # wp2 = wp*wp
 # cvec[ikp]   = np.sum(y*cwt[:,iwp]/s2)
 # cvec[ikp+1] = np.sum(y*swt[:,iwp]/s2)
 # iw = 0
 # for ik in range(0,np2,2):
 #  #cosine column
 #  hes[ikp,ik]   = hnow_cc[iw,iwp]
 #  hes[ikp,ik+1] = hnow_sc[iw,iwp]
 #  #sine column
 #  hes[ikp+1,ik]   = hnow_cs[iw,iwp]
 #  hes[ikp+1,ik+1] = hnow_ss[iw,iwp]
 #  #priors
 #  if (ik == ikp):
 #   hes[ikp,ik] = hnow_cc[iw,iwp] +  np.sqrt(w2_integrate)/(2*rms*w2[iwp]) 
 #   hes[ikp+1,ik+1] = hnow_ss[iw,iwp] + np.sqrt(w2_integrate)/(2*rms*w2[iwp]) 
 #  iw = iw + 1  
 # iwp = iwp + 1
 #t2 = time.time()
 #print('time2',t2-t1)
 
 #print('hes')
 #for i in range(5):
 # print(hes[i,:5])
 # 
 # 
 #print('hes2')
 #for i in range(5):
 # print(hes2[i,:5])
 #input()
 
 #invert the matrix to get the covariance matrix
 cov = np.linalg.inv(hes)
 

 
 #find the parameters
 parm  = cov.dot(cvec)
 skout = parm[1::2]
 ckout = parm[0::2]
 ymodout = np.dot(swt,skout) + np.dot(cwt,ckout)
 var = np.sum((ymodout - y)**2)/(ny - 1)
 
 if (type(si) != np.ndarray):
  cov = cov * var

 
 #monte carlo sample the fit to estimate uncertaintie
 nits = 2000
 cwhires = np.ones((ntgrid,nw))
 swhires = np.ones((ntgrid,nw))
 for iw in range(nw):
  cwhires[:,iw] = np.cos(w[iw]*tgrid)
  swhires[:,iw] = np.sin(w[iw]*tgrid)
 parmnow = np.random.multivariate_normal(parm,cov,size=nits)
 ygridsave = np.zeros((ntgrid,nits))
 for i in range(nits):
  parmnew = parmnow[i,:]
  cknew = parmnew[0::2]
  sknew = parmnew[1::2]
  ygridsave[:,i] = np.dot(swhires,sknew) + np.dot(cwhires,cknew)
 ygridsave = ygridsave*ystd + ymean 
 ygridlo,ygrid,ygridhi = np.percentile(ygridsave,[15.865,50,84.135],axis=1)      
 ygridsd = (ygridhi-ygridlo)/2
 
 ygridop = np.array([ygridlo,ygrid,ygridhi]).T
 
 
 #fit stat
 ymod_itp = np.interp(ti,tgrid,ygrid)
 rcoef = np.corrcoef(ymod_itp,yi)[0,1]
 
 #import matplotlib.pylab as plt
 #plt.clf()
 #fig = plt.figure()
 #ax1 = fig.add_subplot(111)
 #ax1.scatter(t,yi,color='r')
 #ax1.plot(tgrid,ygridop[:,1],color='b')
 #ax1.fill_between(tgrid,ygridop[:,0],ygridop[:,2],alpha=0.3,color='b')
 #ax1.set_xlabel('time')
 #plt.show()
 #plt.clf()
 #np.savetxt('rw_mod_bad_series.dat',np.array([t,s,y]).T)

 return(tgrid,ygridop,f,ckout,skout,rcoef)
 
 
 
 





#test

#from mylcgen import *
#from myrandom import *
#import myresample as mrs
#timeall = []
#sigall  = []
#yall    = []
#
#nlc = 1
#
#tlo = 0.0
#thi = 100
#dtave = 1.0
#dtres = 0.1
#
#for i in range(nlc):
# #generate test light curve and add noise
# datpre      = mylcgen(tlo=tlo,thi=thi,dt=dtres,iseed=132423)
# npre     = np.shape(datpre[:,0])[0]
# datmean  = np.std(datpre[:,1])
# snow =  np.ones(npre)/10*datmean 
# dat = mrs.myresample(dir='',fname=[''],dtave=dtave,sampmin=0.5,sampcode=3,datin=np.array((datpre[:,0],datpre[:,1],snow)).T)
# ndat = np.shape(dat[:,0])[0]
# sig = dat[:,2]
# for i in range(ndat):
#  dat[i,1] = normdis(1,dat[i,1],sig[i])[0]
# sigall.append( sig )
# yall.append( dat[:,1] +50 )
# timeall.append( dat[:,0] )
#
#
#y = yall[0]
#t = timeall[0]
#s = sigall[0]
#
##dat =mylcgen(tlo=tlo,thi=thi,dt=dtave,iseed=132423)
##nd = np.shape(dat[:,0])[0]
##timeall = [dat[:,0]]
##yall = [dat[:,1]]
##sigall = [np.ones(nd)/10*datmean ]
#
#print('going in')
#forecast = 0
#dtm = 0.01
#tmod = np.arange(tlo-forecast,thi+forecast,dtm)
#
##dat = np.loadtxt('/Users/david/github_datascience/vortexa/analytics/arbitrage_flow_study/rw_mod_bad_series.dat')
##t = dat[:,0]
##y = dat[:,1]
##s = dat[:,2]
##tlo = np.min(t)
##thi = np.max(t)
##tmod = np.arange(tlo,thi,10.)
#fc = np.arange(1./100,1./4,1./100)
##fc = np.array([1./10,1./20.,1./40,1./1000,1./30,1./5])
#
#tg,xg,w,ckout,skout,rcoef = rw(t,y,si=s,tgi = tmod,custom_freqs=fc)
#
#
#fig = plt.figure()
#ax1 = fig.add_subplot(211)
#ax1.errorbar(t,y,s)
#ax1.plot(tg,xg[:,1])
##ax1.fill_between(tg,xg[:,0],xg[:,2],alpha=0.5)
#
#ax1 = fig.add_subplot(212)
#ax1.plot(w,ckout**2+skout**2)
#ax1.set_xscale('log')
#ax1.set_yscale('log')
#
#plt.show()
###
##


#this should be here already

def ioslinfit(dat,sig,modin,cspec=0.0,sspec=0.0, specgen = 1.0, rms_pri = 0.0):
 
 #cspec=1 fix offset to mean of data
 #.......=2 ............. median
 #sspec = 1 ...force s to rms of data
 #sspec = 2 force sspec to be 1
 
 
 #if (cspec == 1):
  
 if (specgen == 1):
  mod = modin - np.mean(modin)
 else:
  mod = 1*modin
 
 n=100
 s = 1.0
 c = 1.0
 for i in range(n):
  
  top = np.sum( (dat-s*mod)/sig**2 )
  bot = np.sum( 1./sig**2 )
  c = top/bot
  #print s, c, 'myios.iosinterp'
  sigc = 1/np.sqrt(bot)
  if (cspec==2):
   c = np.median(dat)
  elif (cspec ==1):
   c = np.mean(dat)
  elif (cspec < 0):
   c = np.abs(cspec)
  
  top = np.sum( (dat-c)*mod/sig**2 )
  bot = np.sum( mod**2/sig**2 )
  s = top/bot
  if (sspec==2):
   s = np.abs(s)
  elif (sspec ==1):
   s = np.std(dat)/np.std(mod)
  elif (sspec < 0):
   s = np.abs(sspec)
  
  sigs = 1/np.sqrt(bot)
  
  if (rms_pri == 1):
   modnew = s*mod + c
   s = s*np.std(dat)/np.std(modnew)
  
  
  #print i, s, c,'asdkjsadja'
  
 return(s,sigs,c,sigc) 
  



### python implementation of iterated optimal scaling
#inputs: t: time array
#        x: data array
#     xsig: error bar array
#        w: angular frequency array
#  sk1 ck1: best guess for inut sine and cosine 

#outputs: p[NW,2] sine and cosine amplitudes
#      sigp[NW,2] uncertainties on sine and cosine amplitudes  


import numpy as np
from pylab import *


def ios(t,x,xsig,w,sk1,ck1):
    overflow=1.e10

    ck=1.*ck1
    sk=1.*sk1
    nt=x.shape[0]
    nw=w.shape[0]
    xmean=np.mean(x)
    sigsq=1.*xsig*xsig
    p=np.zeros((nw,2))
    sigp=np.zeros((nw,2))
    nits=100
##### subtract the mean and return this as a constant offset
    xdup=x-xmean
    cw=np.zeros((nt,nw))
    sw=np.zeros((nt,nw))
    for it in range(nt):
        cw[it,:]=np.cos(t[it]*w[:])
        sw[it,:]=np.sin(t[it]*w[:])
    
   
## sin parameter    
    for iw in range(nw):
        for iteration in range(nits):
            top=np.sum((xdup[:]-ck*cw[:,iw])*sw[:,iw]/sigsq[:])
            bot=np.sum(sw[:,iw]*sw[:,iw]/sigsq[:])
            
            if (top == 0):
             sk = 0
            else:
             sk=top/bot
            
            sigsk=np.sqrt(1./bot)
            #print top,bot,xdup.sum(),ck
            #print iteration,iw
            #print cw[:10,iw]
            #print sw[:10,iw]
            #print xdup[:10]
            #print sigsq[:10]
            #print np.sum((xdup[:]-ck*cw[:,iw])*sw[:,iw]/sigsq[:])
            #print sigsq[:]
            #raw_input()
### cos parameter
            top=np.sum((xdup[:]-sk*sw[:,iw])*cw[:,iw]/sigsq[:])
            bot=np.sum(cw[:,iw]*cw[:,iw]/sigsq[:])
            ck=top/bot
            sigck=np.sqrt(1./bot)
            
            sumold=ck+sk
            if iteration > 1:
                convtest=abs((sk+ck - sumold)/sumold)
                if convtest < 0.0001:
                    break
        
        xdup=xdup-sk*sw[:,iw]-ck*cw[:,iw]   # subtract next frequency             
        if abs(sk) > overflow:
            sk=0
       
        if abs(ck) > overflow:
            ck=0
            
        #print iw,w[iw],sk,ck,top
        #print ''
        #raw_input('space')
        p[iw,0]=sk
        p[iw,1]=ck
        sigp[iw,0]=sigsk
        sigp[iw,1]=sigck        
        
        p[-1.:]=0.0
        sigp[-1,:]=0.0
        #print iw, p[iw,:],sigp[iw,:],'myios.py',np.sqrt(sigsq[0]/np.sum(np.sin(w[iw]*t[:])**2)),w[iw],np.sum(np.sin(w[iw]*t[:])**2)/nt           
    return(p,sigp)
    
    
    
    


#!!!!!!!!!!!!!!!! test below
#twopi=np.pi*2
#w0=2*np.pi*0.5
#tlo=0.0
#thi=30.0
#dt=0.1
#t=np.arange(tlo,thi+dt,dt)
#nt=t.shape[0]
#
#tmean=15.0
#tsd=5.0
#x=1/np.sqrt(twopi)/tsd* exp(-0.5*((t-tmean)/tsd)**2)
#
##3.2*np.sin(w0*t)
#
#
#xmean=x.mean()
#xsig=np.ones(nt)*1.0
#sk1=1.0
#ck1=1.0
#wlo=twopi*1./(t[-1]-t[0])
#whi=twopi*1./(2.*dt)
#nw=int(np.ceil(0.5*nt))
#dw=(whi-wlo)/(nw-1)
#w=wlo+np.arange(nw)*dw
#p=np.zeros((nw,2))
#sigp=np.zeros((nw,2))
#
#p,sigp=ios(t,x,xsig,w,sk1,ck1)
#
#tmodlo=t[0]
#tmodhi=t[-1]
#dtmod=dt/10
#ntmod=int(np.ceil((tmodhi-tmodlo)/(dtmod))+1)
#tmod=tmodlo+np.arange(ntmod)*dtmod
#xmod=np.zeros(ntmod)
#for i in range(ntmod):
#    xmod[i]=xmean+np.sum(p[:,0]*np.sin(w[:]*tmod[i])+p[:,1]*np.cos(w[:]*tmod[i]))
#
#plot(t,x,linestyle='none',marker='o')
#plot(tmod,xmod)
#show()
#


### subroutine to call a light curve and construct a power spectrum from the data
# NOTE when specifying the w(iw) = wlo + dw*(iw-1), the -1 is very important, if you dont include it, the model will not fit the data very well, (see you 1st year report for an example of what happens if you leave off the - 1.

# snippet from fortran code !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#NW=ceiling(0.5*NT)
#whi=pi/dt
#wlo=twopi*1/tlen!*0.01
#dw=(whi-wlo)/(NW-1)
#wlo=dw
#NW=ceiling((whi-wlo)/dw)-2


#allocate(w(NW),dtau(NW),sigdtau(NW))
#do i=1,NW
#w(i)=wlo+dw*(i-1)
#enddo
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

import numpy as np
import os
import pylab as plt
from myoptscal import *

pwd = os.getcwd()
print 'Current directory :- ', pwd
dir = raw_input('Please input directory :- ')
os.chdir(dir)

nfiles = np.int(raw_input('Please input number of LC to analyse :- '))
fname=[]
for i in range(nfiles):
    print 'Please enter file',i+1,'of',nfiles,' :- '
    file = raw_input()
    fname.append(file)



#### hardwired parms


nalong=1
include_phase = 0 # 0 or 1 depending if you want to include the phase spectrum in the plot
if (include_phase == 0):
    nvert = 2
else:
    nvert = 3

# set up the plot window
properdtmin = 0.2
fontsize=12


autoylim = 0
ylim=[-6,3]

col=['r','b','g','c','o','p','k','y']
fig=plt.figure()
fig.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=0.5, hspace=0.3)
font = {'family' : 'normal',
        'weight' : 'normal',
        'size'   : fontsize}

plt.rc('font', **font)
loge10 = np.log(10)

##### end of hardwired parms

for i in range(nfiles):
    dat =np.loadtxt(fname[i])
    t   =dat[:,0]
    thi =t.max()
    tlo = t.min()
    tlen = thi - tlo
    x   =dat[:,1]
    sig =dat[:,2]
#    sig = sig*0 + 0.1
    nx  = x.shape[0]
    nxmod = nx*10
    dtmod = (t[-1]-t[0])/(nxmod-1)
    tmod=np.arange(t[0],t[-1]+dtmod, dtmod)
    nxmod=tmod.shape[0]
    
    xop = np.zeros(nxmod)
    sop=[]
    sigsop=[]
    cop=[]
    sigcop=[]
    ampspec=[]
    sigampspec=[]
    phasespec=[]
    sigphasespec=[]
    
#specify the lower and upper frequency limits
    dt=t[1:] - t[:-1]
    dt=dt[np.nonzero(dt)] #problem fixed if 2 datapoints taken at same time
    dtmin = dt.min() #this is the interval between points

# this fixes data points that are super close together making the nyquist frequency higher than can be computed in a reasonable time.    
    if (dtmin < properdtmin):
        print 'Time resolution too high, using average time resolution...'
        dtmin = (tlen/(nx-1))
        if (dtmin < properdtmin):
            print 'The time resolution on this lightcurve is better than sense! I dont like it and will use',properdtmin,'instead.'
            dtmin = properdtmin
    


    nf  = np.int( np.ceil(0.5*nx) )
    fhi = 0.5/dtmin              
    flo = 1/(t[-1]-t[0])
    df = 0.5*(fhi - flo)/(nf-1)
    f = np.arange(flo,fhi+df,df)
    nf = f.shape[0]
    
    w = 2*np.pi*f
# subtract the mean from the data (to avoid worry about the constant offset
    xmean = np.median(x)
    xinp = x - xmean
    xop = np.ones(nxmod)*xmean
    
    
    for i_f in range(nf):
        optscalop = myoptscal(t,xinp,sig,f[i_f])
        s    = optscalop[0]
        sigs = optscalop[1]
        c    = optscalop[2]
        sigc = optscalop[3]
        sop.append(s)
        sigsop.append(sigs)
        cop.append(c)
        sigcop.append(sigc)
        

             
#these bits are updated every i_f
        fit = s*np.sin(w[i_f]*t) + c*np.cos(w[i_f]*t)
        xinp = xinp - fit
        fitmod = s*np.sin(w[i_f]*tmod) + c*np.cos(w[i_f]*tmod)
        
#        print fitmod.mean(),i_f,'test',np.std(fitmod)
# if you fit to too high frequencies, the model goes to crazy high values, introduce a check for this below.
        if (np.abs( np.std(fitmod) ) > 100*xmean):
            print 'The fit has gone crazy! you have tried to fit to too high a frequency... stopping here.'
            nf = len(ampspec)
            break

# amplitude spectrum        
        sigssq = 2*s*sigs
        sigcsq = 2*c*sigc
        a =s*s+c*c
        siga = np.sqrt(sigssq*sigssq + sigcsq*sigcsq)
        ampspec.append(np.sqrt(a))
        sigampspec.append( 0.5/np.sqrt(a)* siga )

# phase spectrum
        soverc    = s/c
        sigsoverc = soverc * np.sqrt( (sigs/s)**2 + (sigc/c)**2 )
        phasespec.append( np.arctan(-1.*soverc) )
        sigphasespec.append( np.abs( sigsoverc/(1. + soverc*soverc) ) ) 

# update the plot model
        xop = xop + fitmod
        


# fit a broken power law to the amplitude spectrum
    ampspeclog = np.log10(np.array(ampspec))
    sigampspeclog = np.abs( np.array(sigampspec)/(np.array(ampspec)*loge10 ) )
    
#    dat=hesfit(x,y,sig,p):
#    dat[0][0]

#this is for the plotting
    ax1=fig.add_subplot(nvert,nalong,0)             # plot the light curve and model fit
    ax1.set_xlabel('time (days)')
    ax1.set_ylabel(r'f$_{\nu}$(t)')
    ax1.errorbar(t,x,sig,linestyle='None',color=col[i])
    ax1.plot(tmod,xop,color=col[i])
    
    ax1=fig.add_subplot(nvert,nalong,1)             # plot the amplitude spectrum
    ax1.set_xlabel('frequency (cyc/day)')
    ax1.set_ylabel(r'$Amp^2$')
#    ax1.set_yscale('log', basey=10)
    ax1.set_xscale('log', basex=10)
    ax1.errorbar(f[:nf],ampspeclog,sigampspeclog,linestyle='None',color=col[i])
    if (autoylim == 0):
        ax1.set_ylim(ylim[0],ylim[1])
    
    
    if (include_phase == 1):
        ax1=fig.add_subplot(nvert,nalong,2)             # plot the phase spectrum
        ax1.set_xlabel('frequency (cyc/day)')
        ax1.set_ylabel('phase')
        ax1.errorbar(f[:nf],phasespec,sigphasespec,linestyle='None',color=col[i])

ax1.legend(fname)

	# save the output
opdat = np.zeros((nf,3))
opdat[:,0]=np.log10(f[:nf])
opdat[:,1]=ampspeclog
opdat[:,2]=sigampspeclog
np.savetxt('mypowerspecop_'+fname[i],opdat)

os.chdir(pwd) #change back to initial directory    




fig.show()

#fig.savefig('mypowerspecfig.png')
       
    

#    ax1=fig.add_subplot(nalong,nvert,idxinc)
#    ax1.set_ylabel(r'$\psi$($\tau$ | $\lambda$)')
#    ax1.set_xlabel(r' $\tau$ (days)')
#    ax1.errorbar(t,x,sig,linestyle=None,color=col[i])





# script to convert differences in magnitudes to ratio of fluxes

import numpy as np

## Input file
inputfile=raw_input('enter input file name...')
print ''
band = raw_input('Enter sloan band (u,g,r,i,z)...')
skiprows=0


def dmag2ratf(dmag):
    ratf=10**(-0.4*dmag)


def make_csv(infile, output, separator=',', add_string=False):
    """
    Takes an input file, strips it of all empty spaces and turns it into a comma separated file
    @type infile: file name
    @type output: file name
    @param infile: name of input file
    @param output: name of output file
    @type separator: String
    @param add_string: Field Separator for output file (defaul is comma)
    @type add_string: False/String
    @param add_string: Enter any string you might want to add to the beginning of each line, default is False
    @note: This function will happily overwrite any important file you give as output!
    """
    print 'Input File is: ' + infile
    print 'output File is: ' + output
    if add_string:
        print 'Adding' + add_string
    f = open(infile, 'r')
    out = open(output, 'w')
    for line in f.readlines():
        tmp = ''
        if line[0] == '#':
            pass
        else:
            if add_string:
                out.write(add_string)
            for x in line.split():
                tmp += x.strip() + separator
            out.write(tmp[:-1] + '\n')
    out.close()
    print "Done!"





# python script to convert swift light curves into mJy
# from erg/s/cm2/A
# NOTE if unknown units, anything ~ 10^-14 is most likely in erg/s/cm^2/A (at least for 5548)

import numpy as np


def erg2mjy(fname,convwav):
 dat=np.loadtxt(fname)
 x=dat[:,1]
 sigx=dat[:,2]
 
 
 conv=2.99292458e-5
 
 #see astro.wku.edu/strolger/UNITS.txt  the 1000 goes from jansky to mJy
 a=convwav*convwav/conv * 1000
 
 x=x*a
 sigx=sigx*a
 # save data
 
 dat[:,1]=x
 dat[:,2]=sigx
 
 # save the converted data
 np.savetxt('converted_'+str(fname),dat)
 



def mjy2erg_now(fmjy,wav):

 conv=wav*wav/2.99292458e-5
 
 op = fmjy/conv/1000.
 
 return(op)
 
#input ferg = ferg / 1e-15
def erg2mjy_now(ferg,wav):

 conv=wav*wav/2.99292458e-5
 
 op = ferg*1000*conv/1.e15
 
 return(op)


#import mean flux in mjy
import numpy as np

fluxmean = 2.0

wav = 5000.





def abmag(fmjy):
 return(-2.5 * np.log10(fmjy/1000.) + 8.9)


def absmag(abm,dMpc):
 return( abm - 5*np.log10(dMpc) - 25. )

#uses empirical results obtained by Macleod, Ivezic et al 2010 
def drwtau_mc(embh,wav, absmag):
 A = 2.4
 B = 0.17
 C = 0.03
 D = 0.21
 taulog = A + B*np.log10(wav/4000.) + C*(absmag + 23.) + D*np.log10(embh/1.e9)
 return(10**taulog)




def drwsfinf_mc(embh, wav, absM, z = 0.):
 #! uses empirical results obtained by Macleod, Ivezic et al 2010 (DIFFERENT NUMBERS FROM drwtau_mc.f90)
 A = -0.56
 B = -0.479
 C = 0.111
 D = 0.11
 E = 0.07
 sfinflog = A + B*np.log10(wav/4000.) + C*(absM + 23.) + D*np.log10(embh/1.e9) + E*np.log10(1.+z)
 return(10**sfinflog)





def mfamp(embh,wav,fmjy,tlen,dMpc, z= 0.0):
 abm    = abmag(fmjy)
 absM   = absmag(abm,dMpc)
 taumc  = drwtau_mc(embh,wav,absM)
 sfinf  = drwsfinf_mc(embh, wav, absM, z = z)
 if taumc == 0:
  sf_inf = sfinf
 else:
  sf_inf = sfinf * np.sqrt( (1. - np.exp(-tlen/taumc)) )

 sdmag = np.abs(sf_inf)/np.sqrt(2)
 #this is the variance in ab magnitudes. Need to change to fluxes
 a = 1./1000/3631
 sdmjy = np.abs( -0.4 * 10**(-abm*0.4) / a * np.log(10.)  * sdmag )
 
  
 #print('wav, mean_AB, rms_AB ', wav, abm, sdmag,'  <-- report myfake_amp --> mean_mJy, rms_mJy ',fmjy, sdmjy)
 
 return(sdmjy)
 
 
 
 
 
 
 
 
 


import numpy as np
import matplotlib.pyplot as pl
from matplotlib.ticker import MaxNLocator
from matplotlib.colors import LinearSegmentedColormap
from matplotlib.patches import Ellipse
import matplotlib.cm as cm
import scipy
import scipy.stats

def corner_1(xs, weights=None, labels=None, extents=None, truths=None,
           truth_color="#4682b4", scale_hist=False, quantiles=[],
           verbose=True, plot_contours=False, plot_datapoints=False,
           fig=None,header=[],sigconts=[100.-68,100.-95,100-99.7],figname='histplot.pdf',skip=1,xti=[-1],yti=[-1], annotate=[],plot_corr=0,**kwargs):
    
    npar = np.shape(xs)[1]
    if (len(header) ==0):
     header=['']*npar
    
    """
    Make a *sick* corner plot showing the projections of a data set in a
    multi-dimensional space. kwargs are passed to hist2d() or used for
    `matplotlib` styling.

    Parameters
    ----------
    xs : array_like (nsamples, ndim)
        The samples. This should be a 1- or 2-dimensional array. For a 1-D
        array this results in a simple histogram. For a 2-D array, the zeroth
        axis is the list of samples and the next axis are the dimensions of
        the space.

    weights : array_like (nsamples,)
        The weight of each sample. If `None` (default), samples are given
        equal weight.

    labels : iterable (ndim,) (optional)
        A list of names for the dimensions.

    extents : iterable (ndim,) (optional)
        A list where each element is either a length 2 tuple containing
        lower and upper bounds (extents) or a float in range (0., 1.)
        giving the fraction of samples to include in bounds, e.g.,
        [(0.,10.), (1.,5), 0.999, etc.].
        If a fraction, the bounds are chosen to be equal-tailed.

    truths : iterable (ndim,) (optional)
        A list of reference values to indicate on the plots.

    truth_color : str (optional)
        A ``matplotlib`` style color for the ``truths`` makers.

    scale_hist : bool (optional)
        Should the 1-D histograms be scaled in such a way that the zero line
        is visible?

    quantiles : iterable (optional)
        A list of fractional quantiles to show on the 1-D histograms as
        vertical dashed lines.

    verbose : bool (optional)
        If true, print the values of the computed quantiles.

    plot_contours : bool (optional)
        Draw contours for dense regions of the plot.

    plot_datapoints : bool (optional)
        Draw the individual data points.
    
    annotate : list[[...nj]....ni] of strings to add to each plot
    
    sigconts : plot contour levels by default at 1, 2, 3 sigma... e.g [100.-68,100.-95,100-99.7]
    
    fig : matplotlib.Figure (optional)
        Overplot onto the provided figure object.
    
    plot_corr : 1 to annotate plots with correlation coefficient

    """
    nconts = len(sigconts)
    
    
    # Deal with 1D sample lists.
    xs = np.atleast_1d(xs)
    if len(xs.shape) == 1:
        xs = np.atleast_2d(xs)
    else:
        assert len(xs.shape) == 2, "The input sample array must be 1- or 2-D."
        xs = xs.T
    #assert xs.shape[0] <= xs.shape[1], "I don't believe that you want more " \
    #                                   "dimensions than samples!"

    if weights is not None:
        weights = np.asarray(weights)
        if weights.ndim != 1:
            raise ValueError('weights must be 1-D')
        if xs.shape[1] != weights.shape[0]:
            raise ValueError('lengths of weights must match number of samples')

    # backwards-compatibility
    plot_contours = kwargs.get("smooth", plot_contours)

    K = len(xs)
    #print(K)
    factor = 1.0*6/K           # size of one side of one panel
    lbdim = 0.9 * factor   # size of left/bottom margin
    trdim = 0.05 * factor  # size of top/right margin
    whspace = 0.05         # w/hspace size
    plotdim = factor * K + factor * (K - 1.) * whspace
    dim = lbdim + plotdim + trdim

    if fig is None:
        fig, axes = pl.subplots(K, K, figsize=(dim, dim))
    else:
        try:
            axes = np.array(fig.axes).reshape((K, K))
        except:
            raise ValueError("Provided figure has {0} axes, but data has "
                             "dimensions K={1}".format(len(fig.axes), K))
    lb = lbdim / dim
    tr = (lbdim + plotdim) / dim
    fig.subplots_adjust(left=lb, bottom=lb, right=tr, top=tr,
                        wspace=whspace, hspace=whspace)

    if extents is None:
        extents = [[x.min(), x.max()] for x in xs]

        # Check for parameters that never change.
        m = np.array([e[0] == e[1] for e in extents], dtype=bool)
        if np.any(m):
            raise ValueError(("It looks like the parameter(s) in column(s) "
                              "{0} have no dynamic range. Please provide an "
                              "`extent` argument.")
                             .format(", ".join(map("{0}".format,
                                                   np.arange(len(m))[m]))))
    else:
        # If any of the extents are percentiles, convert them to ranges.
        for i in range(len(extents)):
            try:
                emin, emax = extents[i]
            except TypeError:
                q = [0.5 - 0.5*extents[i], 0.5 + 0.5*extents[i]]
                extents[i] = quantile(xs[i], q, weights=weights)

    for i, x in enumerate(xs):
        ax = axes[i, i]
        # Plot the histograms.
        n, b, p = ax.hist(x, weights=weights, bins=kwargs.get("bins", 50),
                          range=extents[i], histtype="step",
                          color=kwargs.get("color", "k"))
        
        ax.set_title(header[i])
        xlimlo = extents[i][0]
        xlimhi = extents[i][1]
        binwidth_x = (xlimhi-xlimlo)/kwargs.get("bins", 50)
        
        if (xti == []):
         ax.set_xticklabels([])
        if (yti == []):
         ax.set_yticklabels([])
        
        if truths is not None:
            ax.axvline(truths[i], color=truth_color)

        # Plot quantiles if wanted.
        if len(quantiles) > 0:
            qvalues = quantile(x, quantiles, weights=weights)
            for q in qvalues:
                ax.axvline(q, ls="dashed", color=kwargs.get("color", "k"))
            if verbose:
                print("Quantiles:")
                print(zip(quantiles, qvalues))

        # Set up the axes.
        ax.set_xlim(extents[i])
        if scale_hist:
            maxn = np.max(n)
            ax.set_ylim(-0.1 * maxn, 1.1 * maxn)
        else:
            ax.set_ylim(0, 1.1 * np.max(n))
        ax.set_yticklabels([])
        ax.xaxis.set_major_locator(MaxNLocator(5))

        # Not so DRY.
        if i < K - 1:
            ax.set_xticklabels([])
        else:
            [l.set_rotation(45) for l in ax.get_xticklabels()]
            if labels is not None:
                ax.set_xlabel(labels[i])
                ax.xaxis.set_label_coords(0.5, -0.3)

        for j, y in enumerate(xs):
            ax = axes[i, j]
            if j > i:
                ax.set_visible(False)
                ax.set_frame_on(False)
                continue
            elif j == i:
                continue

            hist2d(y[::skip], x[::skip], ax=ax, extent=[extents[j], extents[i]],
                   plot_contours=plot_contours,
                   plot_datapoints=plot_datapoints,
                   weights=weights, **kwargs)
            ylimlo = extents[j][0]
            ylimhi = extents[j][1]
            binwidth_y = (ylimhi-ylimlo)/kwargs.get("bins", 50)

            both = np.array((x,y)).T
            pdf1=scipy.stats.kde.gaussian_kde(both.T)
            #print(ylimlo,ylimhi,binwidth_y,xlimlo,xlimhi,binwidth_x,extents)
          
            q,w=np.meshgrid(np.arange(xlimlo,xlimhi,binwidth_x), np.arange(ylimlo,ylimhi,binwidth_y))
            r1=pdf1([q.flatten(),w.flatten()])
            s1=[]
            
            for ic in range(nconts):
             s1.append(scipy.stats.scoreatpercentile(pdf1(pdf1.resample(1000)), sigconts[ic]))
            
            r1.shape=(q.shape[0],q.shape[1])
            for ic in range(nconts):
             ax.contour(np.arange(xlimlo,xlimhi,binwidth_x), np.arange(ylimlo,ylimhi,binwidth_y), r1, [s1[ic]])
             
            if (xti == []):
             ax.set_xticklabels([])
            if (yti == []):
             ax.set_yticklabels([])
            
            if (plot_corr > 0):
             r_ann = np.corrcoef(np.array((y[::skip],x[::skip])))[0,1]
             nt = np.shape(y)[0]
             #for i in range(nt):
             # print(y[i],x[i])
             #print(r_ann)
             #print(np.corrcoef(np.array((y[1:],x[1:]))))
             #raw_input()
             ax.text(0.95,0.9,'R='+str(np.round(r_ann,2)),ha='right',transform=ax.transAxes)
            
            if (len(annotate) > 0):
             ax.text(0.95,0.9,annotate[i][j],ha='right',transform=ax.transAxes)
             
            if truths is not None:
                ax.plot(truths[j], truths[i], "s", color=truth_color)
                ax.axvline(truths[j], color=truth_color)
                ax.axhline(truths[i], color=truth_color)

            ax.xaxis.set_major_locator(MaxNLocator(5))
            ax.yaxis.set_major_locator(MaxNLocator(5))

            if i < K - 1:
                ax.set_xticklabels([])
            else:
                [l.set_rotation(45) for l in ax.get_xticklabels()]
                if labels is not None:
                    ax.set_xlabel(labels[j])
                    ax.xaxis.set_label_coords(0.5, -0.3)

            if j > 0:
                ax.set_yticklabels([])
            else:
                [l.set_rotation(45) for l in ax.get_yticklabels()]
                if labels is not None:
                    ax.set_ylabel(labels[i])
                    ax.yaxis.set_label_coords(-0.3, 0.5)

    

     
    if (figname != ''):
     pl.tight_layout()
     pl.savefig(figname)
    return fig,axes


def quantile(x, q, weights=None):
    """
    Like numpy.percentile, but:

    * Values of q are quantiles [0., 1.] rather than percentiles [0., 100.]
    * scalar q not supported (q must be iterable)
    * optional weights on x

    """
    if weights is None:
        return np.percentile(x, [100. * qi for qi in q])
    else:
        idx = np.argsort(x)
        xsorted = x[idx]
        cdf = np.add.accumulate(weights[idx])
        cdf /= cdf[-1]
        return np.interp(q, cdf, xsorted).tolist()


def error_ellipse(mu, cov, ax=None, factor=1.0, **kwargs):
    """
    Plot the error ellipse at a point given its covariance matrix.

    """
    # some sane defaults
    facecolor = kwargs.pop('facecolor', 'none')
    edgecolor = kwargs.pop('edgecolor', 'k')

    x, y = mu
    U, S, V = np.linalg.svd(cov)
    theta = np.degrees(np.arctan2(U[1, 0], U[0, 0]))
    ellipsePlot = Ellipse(xy=[x, y],
                          width=2 * np.sqrt(S[0]) * factor,
                          height=2 * np.sqrt(S[1]) * factor,
                          angle=theta,
                          facecolor=facecolor, edgecolor=edgecolor, **kwargs)

    if ax is None:
        ax = pl.gca()
    ax.add_patch(ellipsePlot)

    return ellipsePlot


def hist2d(x, y, *args, **kwargs):
    """
    Plot a 2-D histogram of samples.

    """
    ax = kwargs.pop("ax", pl.gca())

    extent = kwargs.pop("extent", [[x.min(), x.max()], [y.min(), y.max()]])
    bins = kwargs.pop("bins", 50)
    color = kwargs.pop("color", "k")
    linewidths = kwargs.pop("linewidths", None)
    plot_datapoints = kwargs.get("plot_datapoints", True)
    plot_contours = kwargs.get("plot_contours", True)

    cmap = cm.get_cmap("gray")
    cmap._init()
    cmap._lut[:-3, :-1] = 0.
    cmap._lut[:-3, -1] = np.linspace(1, 0, cmap.N)

    X = np.linspace(extent[0][0], extent[0][1], bins + 1)
    Y = np.linspace(extent[1][0], extent[1][1], bins + 1)
    try:
        H, X, Y = np.histogram2d(x.flatten(), y.flatten(), bins=(X, Y),
                                 weights=kwargs.get('weights', None))
    except ValueError:
        raise ValueError("It looks like at least one of your sample columns "
                         "have no dynamic range. You could try using the "
                         "`extent` argument.")

    V = 1.0 - np.exp(-0.5 * np.arange(0.5, 2.1, 0.5) ** 2)
    Hflat = H.flatten()
    inds = np.argsort(Hflat)[::-1]
    Hflat = Hflat[inds]
    sm = np.cumsum(Hflat)
    sm /= sm[-1]

    for i, v0 in enumerate(V):
        try:
            V[i] = Hflat[sm <= v0][-1]
        except:
            V[i] = Hflat[0]

    X1, Y1 = 0.5 * (X[1:] + X[:-1]), 0.5 * (Y[1:] + Y[:-1])
    X, Y = X[:-1], Y[:-1]

    if plot_datapoints:
        ax.plot(x, y, "o", color=color, ms=1.5, zorder=-1, alpha=0.1,
                rasterized=True)
        if plot_contours:
            ax.contourf(X1, Y1, H.T, [V[-1], H.max()],
                        cmap=LinearSegmentedColormap.from_list("cmap",
                                                               ([1] * 3,
                                                                [1] * 3),
                        N=2), antialiased=False)

    if plot_contours:
        ax.pcolor(X, Y, H.max() - H.T, cmap=cmap)
        ax.contour(X1, Y1, H.T, V, colors=color, linewidths=linewidths)

    data = np.vstack([x, y])
    mu = np.mean(data, axis=1)
    cov = np.cov(data)
    if kwargs.pop("plot_ellipse", False):
        error_ellipse(mu, cov, ax=ax, edgecolor="r", ls="dashed")

    ax.set_xlim(extent[0])
    ax.set_ylim(extent[1])








def hist_1(xs, labels=None, lims=None, truths=None,nbins=[50],colin ='b',truth_color = 'r',annotate = None, figname = 'histplot.pdf'):
 
 fig = pl.figure()
 ax1 = fig.add_subplot(111)
 
 if (labels != None):
  ax1.set_xlabel(labels[0])
  
 
 if (lims != None):
  ax1.set_xlim(lims[0])
 
 
 if (len(nbins) > 0):
  bins = np.array(nbins)
 else:
  bins = nbins[0]
 
 if (truths != None):
  
  ylimnew = ax1.get_ylim
  ax1.vlines(truths[0],ylimnew[0],ylimnew[1],col = truth_color)
  
 ax1.hist(xs,bins = nbins[0], histtype='step',color = colin)
 
 if (annotate != None):
  ax1.text(0.95,0.9,annotate,ha='right',transform=ax1.transAxes)
 
 pl.savefig(figname)
 
 return()
  
 
 
def corner(xs, weights=None, labels=None, extents=None, truths=None,truth_color="#4682b4", scale_hist=False, quantiles=[],verbose=True, plot_contours=False, plot_datapoints=False,fig=None,sigconts=[100.-68,100.-95,100-99.7],figname = 'histplot.pdf',annotate=[], **kwargs):


 
 ndim = np.shape(xs)
 
 if (ndim[1] == 1):
  if (len(annotate) > 0):
   text = annotate[0][0]
  hist_1(xs, labels=labels, truths=truths,colin = 'k',figname = figname, truth_color = truth_color, annotate = text)
  
 else:
  corner_1(xs, weights=weights, labels=labels, extents=extents, truths=truths,
           truth_color=truth_color, scale_hist=scale_hist, quantiles=quantiles,
           verbose=verbose, plot_contours=plot_contours, plot_datapoints=plot_datapoints,
           fig=fig,sigconts=sigconts,figname = figname, annotate = annotate, **kwargs)

 return()
  


###T(R) prof


import numpy as np
import const
from pylab import *







r_out=100000
#r_in=1/const.ps*rsch(M)
inc=0
D=1
M=1*10**8
M_dot=1
R_star=3

##Define the array containing the radius and angular information
size=10000
temp_inf=np.zeros((size,2,2))
#delta_r=(r_out-r_in)/size

##set this to 1 if you want to specify more parameters


##text for plots
#delta_r=(r_out-r_in)/size
text=np.zeros(6)
text[0]=D

text[2]=r_out
text[3]=M_dot
text[4]=M
text[5]=inc

##black hole scwarzschild radius
def r_sch(M):
	rs=2*const.G*M/const.c**2
	return(rs)

r_in=100000/const.ps*r_sch(M)
text[1]=r_in
M=M*const.Msun
M_dot=M_dot*const.Msun
D=D*const.c/const.H0
#delta_r=delta_r*const.ps
r_out=r_out*r_sch(M)#const.ps
#r_in=r_in*const.ps
R_0=3.*r_sch(M)#*R_star
r_in=3.*r_sch(M)
delta_r=(r_out-r_in)/size
	
##populate the x and y dimensions of the array with radial and azimuthal coordinates 
temp_inf[:,0,0]=1.*np.arange(size)/size * (r_out-r_in)  +r_in
temp_inf[:,1,0]=1.*np.arange(size)/size * 2*np.pi
#delta_r=(r_out-r_in)/size

## solid angle  ## radii are stored in temp_inf[:,0,0]
s_a=2*np.pi*temp_inf[:,0,0]*delta_r*np.cos(inc)/D**2
	

##define the temperature radius profile for a steady state accretion disk in parsecs
##viscous dissipation due gives rise to a factor of 3/8pi
def T_R(M,M_dot,R,R_star):
	#M=M*const.Msun
	#M_dot=M_dot*const.Msun
	#R=R*const.ps
	#R_0=r_sch(M)*R_star
	T=(3*const.G*M*M_dot*(1-(R_0/R)**0.5)/(8*np.pi*const.sb*R**3))**0.25
	return(T)
	    


	

###Now need to multiply by the solid angle element of each annuli


T=T_R(M,M_dot,temp_inf[:,0,0],R_star)	

##
#define an array which stores wavelengths from the wein temperature of the lowest radi to highest radi

wavmin=0#1*10**-10#const.w_c/T[1]
wavmax=1*10**-3#const.w_c/T[size-1]#1*10**-6#
wavelength=1.*np.arange(size)/size * (wavmax-wavmin) +wavmin   ## store the wavelength ranges over to calculate disk spectrum 	
wavelength = const.w_c/T	

##plank spectrum as a function of wavelength have to sum over temperature contributions from all of disk
B_T=np.zeros(size)
for i in range(size):
	B_T[i]=(2*const.plank*const.c**2/wavelength[i]**3 * (np.exp(const.plank*const.c/(wavelength[i]*const.k*T)) - 1)**-1 * s_a).sum()


	
	
## Just to plot everything
output=1
if output == 1:
	plot(wavelength,B_T)
	xlabel('Wavelength / m')
	ylabel('flux Wm^-1')
	xticks(fontsize=15)
	yticks(fontsize=15)
	title ('Z = '+str(text[0])+' .  r_in/r_out (pc)=3R_s /' +str(text[2])+'.  M='+str(text[4])+'.  M dot='+str(text[3])+'.  Inc='+str(text[5]))
	show()
	


# Function to calculate a skewed gaussian with the mean, sig, skew and kurt inputted
# INPUT: tau[1:Ntau],tau0,sig0,pskew,pkurt
# OUTPUT: psitau[1:Ntau]
import numpy as np

#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  m3 function
def m3(y):
 yy=y*y
 yyy=yy*y
 m3=1./np.sqrt(3.)*(2*yyy-3*y)

 return(m3)
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  m4 function
def m4(y):

 yy=y*y
 yyyy=yy*yy

 m4=1./np.sqrt(24.) * (4*yyyy - 12*yy +3*y)
 
 return(m4)
 
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! tfskewgauss function
def tfskewgauss(tau,p):#tau0,sig0,pskew,pkurt):
 
 psi0 = p[0]
 tau0 = p[1]
 sig0 = p[2]
 pskew = p[3]
 pkurt = p[4]
 
 y = (tau - tau0)/sig0

 psitau = psi0 * np.exp(-y*y/2) * (1. + pskew * m3(y) + pkurt*m4(y) / ( sig0 *np.sqrt(2*np.pi) ) )

 return (psitau)



## tested works 2/10/2014
#
## test the above function below
#
#p0 = 1
#tau0 = 10.0
#sig = 3.0
#skew = 0.2
#kurt = 0.0
#
#tau = np.arange(0,30,0.01)
#psitau = tfskewgauss(tau, (p0,tau0, sig, skew, kurt))
#dtau = tau[1] - tau[0]
#psiarea = np.sum(psitau*dtau)
#top = np.sum(psitau*tau)
#bot = np.sum(psitau)
#taumean = top/bot
#
#from pylab import *
#plot(tau,psitau)
#show()
#
##!! calculate the moments and compare
#import mystatmoments as ms
#
#opsig2 = ms.mystatmoments(tau,psitau/psiarea,taumean,2)
#opskew = ms.mystatmoments(tau,psitau/psiarea,taumean,3)
#opkurt = ms.mystatmoments(tau,psitau/psiarea,taumean,4)





#new sep4 different colours for each legens element unique colour python loop colour 
#plot all light curves with file names listed in fname
#if seperate == 1, plot on seperate graphs, else plot all on same
# if xminforce,xmaxforce entered, force the time limits of the plot

import numpy as np
#import pylab as plt
import os
import matplotlib.pylab as plt
import matplotlib.ticker as mtick

plt.rcParams['axes.linewidth'] = 2
plt.rcParams['font.weight'] = 2
plt.rcParams['xtick.major.width'] = 2
plt.rcParams['ytick.major.width'] = 2
def myplotlc(fname,seperate,xminforce = -1,xmaxforce = -1, wavfile = '',wav_abv_name='', savefigname='',axlab = ['',''],showticks=[1,1],plottype='e',multiplot=0,fig=0,forcecol=[],ann=[],hlin=[],vlin=[],conv2ergs=0,alpha=1.0,linewidth=1.0,ylimforce=[],yt=[-1],fiddle=[]):

 xmintemp = []
 xmaxtemp = []
 nann = len(ann)
 
 with file(fname) as f:
  lines = f.read().splitlines()


 datplot = []
 nf = len(lines)

 include_wavs = 0
 include_wav_abv = 0
 f.close()
 
# deal with wavelength files (if included) and abbreviations (if included) 
 if (len(wavfile) > 0):
  include_wavs = 1
  wav = np.array(np.loadtxt(wavfile))
  a = np.zeros(nf)
  a[:] = 1.*wav
  wav = 1.*a
  
  wav_scaled = (wav - wav.min()) / wav.ptp()
  wav_col    = plt.cm.nipy_spectral(wav_scaled)
 else:#plot according to wavelength or (if not entered) blue (=0.2 with nipy_spectral cmap)
  wav = [1000]*nf
  wav_col    = plt.cm.nipy_spectral(np.linspace(0,1,nf))
 
 if (len(wav_abv_name) > 0):
  f = open(wav_abv_name)
  wav_abv = f.readlines()
  f.close()
  include_wav_abv =1 
 else:
  wav_abv = ['']*nf
 
 
 if (len(forcecol) != 0):
  wav_col = forcecol
 
 
 
 plotlist=[] #list holding ax1.plot command for legend
 for i in range(nf):
  print 'Loading file ',i,' of', nf,'... ',lines[i].strip()
  
  
  print 'argh!!!', lines[i]
  print os.getcwd()
  print ''
  dat = np.loadtxt(lines[i].strip())
  
  if (len(fiddle) > 0):
   dat[:,1] = dat[:,1]*fiddle[i]
   dat[:,2] = dat[:,2]*fiddle[i]
  
  if ((conv2ergs == 1) and (wav[i] > 1.) ):
   #print i
   #print wav
   #print wav[i]
   #raw_input()
   #b = np.float(wav[i])
   #print b
   #raw_input()
   a = 2.9979e-8 *1.e16/ np.float(wav[i])**2
   dat[:,1] = dat[:,1]*a
   dat[:,2] = dat[:,2]*a
  
  
  print 'minimum spacing between points...',np.min(dat[1:,0]-dat[:-1,0])
  print ''
  #print 'diagnose ', len(dat)
  xmintemp.append(np.min(dat[:,0]))
  xmaxtemp.append(np.max(dat[:,0]))
  datplot.append(dat)


#sort the plot limits
 if (xminforce < 0):
  xmin = min(xmintemp)
 else:
  xmin = xminforce

 if (xmaxforce < 0):
  xmax = max(xmaxtemp)
 else:
  xmax = xmaxforce
#
  
 if (multiplot ==0):
  fig = plt.figure()
  
 if (seperate == 0):
  ax1 = fig.add_subplot(1,1,1) 
 
 nhlin = len(hlin) 
 nvlin = len(vlin) 
 l=[]
 for i in range(nf):
  if (seperate ==1):
   ax1 = fig.add_subplot(nf,1,i+1)

## add title
  if (nann > 0):
   ax1.text(ann[i][1],ann[i][2],ann[i][0],ha='left',transform=ax1.transAxes,color = wav_col[i])


## add horrizontal lines
  if (nhlin > 0):
   hlintemp = hlin[i]
   ax1.hlines(hlintemp,xmin = xmin,xmax=xmax,color = wav_col[i])
    
    
## add vertical lines
  if (nvlin > 0):
   vlintemp = vlin[i]
   for xline in vlintemp:
    ab=ax1.get_ylim()
    ax1.vlines(xline,ymin = ab[0],ymax=ab[1],color = wav_col[i])    
#label with wavelengths and abbreviations if present
  
  #if (include_wavs == 1):
  # ax1.text(0.9,0.92,str(int(round(wav[i],0)))+"$\AA$",ha='left',va='top',transform=ax1.transAxes, fontsize = 12)
  #if (include_wav_abv ==1):
  # ax1.text(0.9,0.42,wav_abv[i],ha='left',va='top',transform=ax1.transAxes,fontsize = 12)
  
  if (wav_abv[i] != ''):
   ltemp = wav_abv[i]+"$\AA$"
  else: 
   ltemp = str(int(round(wav[i],0)))+"$\AA$"
  
  
  if (plottype == 'e'):
   plotlist.append( ax1.errorbar(datplot[i][:,0], datplot[i][:,1], datplot[i][:,2],linestyle='None',color = wav_col[i],label = ltemp,alpha=alpha,linewidth=linewidth) )
  elif (plottype == 'l'):
   plotlist.append( ax1.plot(datplot[i][:,0], datplot[i][:,1],marker = 'None', color = wav_col[i],label = ltemp,alpha=alpha,linewidth=linewidth) )
  elif (plottype == 'p'):
   plotlist.append( ax1.scatter(datplot[i][:,0], datplot[i][:,1], color = wav_col[i],marker='o', edgecolors='None', label = ltemp,alpha=alpha,linewidth=linewidth) )
   #ax1.plot(datplot[i][:,0], datplot[i][:,1],linestyle = 'None', color = wav_col[i],marker='o', edgecolors='None')

  #only display xticks on bottom
  #l.append(ltemp)
  
  
  
  ax1.yaxis.set_major_formatter(mtick.FormatStrFormatter('%.1f'))
  #if ((i < nf - 1) or (showticks[0] == 0) ):
  # plt.setp(ax1.get_xticklabels(), visible=False)
  
  if (showticks[1] == 0):
   plt.setp(ax1.get_yticklabels(), visible=False)

  if (i == nf-1):
   ax1.set_xlabel(axlab[0])
  if (i == nf/2):
   if (axlab[1] == ''):
    ax1.set_ylabel(r'$f_\nu ( \lambda )  / 10^{-14} \left( erg s^{-1} cm^{-2}  \AA^{-1} \right)$')
   else:
    ax1.set_ylabel(axlab[1]) 
  #if (i > nf -2):
  # ax1.tick_params(axis='x', pad = 30)
   
  
  ax1.set_xlim(xmin,xmax)
  #ax1.set_xticks(np.linspace(xmin,xmax,4))
  if (seperate ==1):
   ylims = ax1.get_ylim()
   yrange = ylims[1]-ylims[0]
   ax1.set_ylim(ylims[0]-yrange/20,ylims[1]+yrange/20)
   ax1.tick_params(axis='y',labelsize=16)
 
 leg = ax1.legend(numpoints = 1)
 leg.draw_frame(False)
 
 for i in range(nf):
  text = leg.get_texts()[i]
  text.set_color(plotlist[i][0].get_color())

  
 if (len(ylimforce) > 0 ):
  ax1.set_ylim((ylimforce[0],ylimforce[1]))
 
 if (len(yt) > 0):
  if (yt[0] != -1):
   ax1.set_yticks(yt)  
    
 
 #print xmintemp
 #print xmaxtemp
 if (len(savefigname) == 0):
  plt.show()
 elif (savefigname == 'wait'):
  print 'waiting...'
 else:
  plt.tight_layout()
  plt.savefig(savefigname,bbox_inches='tight',orientation='portrait')
  

 return(datplot)
 
 
 
 
 
def saveindir(fname,dir,wavfile='',wav_abv_name=''): 
 seperate = 1
 
 pwd = os.getcwd()
 
 
 with file(fname) as f:
  lines = f.read().splitlines()
 f.close()
 
 nf = len(lines)
 wavfil=['']*nf
 wavabv=['']*nf
 
 if (wavfile != ''):
  with file(wavfile) as f:
   wavfil = f.read().splitlines()
  f.close()
 

 if (wav_abv_name != ''):
  with file(wav_abv_name) as f:
   wavabv = f.read().splitlines()
  f.close()
 
 

 
 for idx in range(nf): 
  wfroutine = ''
  wabvroutine=''
  savefigname='fig_'+str(idx)+'.png'
 



  if (wavfile != ''):
   f = open('wavfiletemp.dat','w')
   f.write(wavfil[idx])
   f.close()
   wfroutine = 'wavfiletemp.dat'
   savefigname = 'fig_'+str(wavfil[idx])+'.png'
 
 
  if (wav_abv_name != ''):
   f = open('wavabvtemp.dat','w')
   f.write(wavabv[idx])
   f.close()
   wabvroutine = 'wavabvtemp.dat'
   savefigname = 'fig_'+str(wavabv[idx])+'.png'
   

  f = open('filetemp.dat','w')
  f.write(lines[idx])
  print 'file ... ',lines[idx]
  f.close()
   
   
  
  if not os.path.exists(dir):
   os.makedirs(dir)
     
  #move temporary files to output directory
  #str_sys  = 'mv '+wfroutine+' '+dir  
  #os.system(str_sys)
  #str_sys  = 'mv '+wabvroutine+' '+dir  
  #os.system(str_sys)
  #str_sys  = 'mv filetemp.dat '+dir  
  #os.system(str_sys)
  
  
  #save the light curve in the temporary directory
  #os.chdir(dir)
  savefigname = dir+'/'+savefigname
  a = myplotlc('filetemp.dat',seperate,xminforce = -1,xmaxforce = -1, wavfile = wfroutine,wav_abv_name=wabvroutine, savefigname=savefigname,axlab = ['',''],showticks=[1,1],plottype='e',multiplot=0,fig=0,forcecol=[],ann=[],hlin=[],vlin=[])
 
  #clean up temporary files
  str_sys  = 'rm '+wfroutine  
  os.system(str_sys)
  str_sys  = 'rm '+wabvroutine 
  os.system(str_sys)
  str_sys  = 'rm filetemp.dat'  
  os.system(str_sys)
  
  #os.chdir(pwd)
  

 return()
  
  
  



import numpy as np
select=np.select
power=np.power
transpose=np.transpose
arange=np.arange

def factor(wl):
    return select(
        [ wl > 700.,
          wl < 420.,
          True ],
        [ .3+.7*(780.-wl)/(780.-700.),
          .3+.7*(wl-380.)/(420.-380.),
          1.0 ] )

def raw_r(wl):
    return select(
        [ wl >= 580.,
          wl >= 510.,
          wl >= 440.,
          wl >= 380.,
          True ],
        [ 1.0,
          (wl-510.)/(580.-510.),
          0.0,
          (wl-440.)/(380.-440.),
          0.0 ] )

def raw_g(wl):
    return select(
        [ wl >= 645.,
          wl >= 580.,
          wl >= 490.,
          wl >= 440.,
          True ],
        [ 0.0,
          (wl-645.)/(580.-645.),
          1.0,
          (wl-440.)/(490.-440.),
          0.0 ] )

def raw_b(wl):
    return select(
        [ wl >= 510.,
          wl >= 490.,
          wl >= 380.,
          True ],
        [ 0.0,
          (wl-510.)/(490.-510.),
          1.0,
          0.0 ] )

gamma = 0.80
def correct_r(wl):
    return power(factor(wl)*raw_r(wl),gamma)
def correct_g(wl):
    return power(factor(wl)*raw_g(wl),gamma)
def correct_b(wl):
    return power(factor(wl)*raw_b(wl),gamma)

ww=arange(380.,781.)
def rgb():
    return transpose([correct_r(ww),correct_g(ww),correct_b(ww)])



#!! program to evaluate the uncertainties and parameter values of a model by minimising chi squared
#!! tested 3rd august with ax**2+bx +c and works (code below) 
#subroutine hesfit(x,y,sig,p,N,NP,pout,sigpout)

#inputs x(N),y(N),sig(N) x y and uncertainty values
# p(N,NP) the pattern to be fitted i.e fitting y=ax^2 + bsin(x) p(1:N,1) would be x**2 and p(1:N,2) would be an array of sin(x)

# outputs: pout(NP), sigpout(NP parameters and uncertainties

import numpy as np






#!!! test this using y=ax + b
#real y(10),x(10),sig(10),p(10,3),pout(3),sigpout(3),ymod(10)
#
#n=10
#n_p=3
#a=1.2
#b=2.8
#c=5.0

#x=np.arange(n)+1
#y=a*x+b*x**2 + c
#sig=np.ones(n)*0.2


#!!! set up patterns
#p=np.zeros((n,n_p))
#hes=np.zeros((n_p,n_p))
#p[:,0]=x
#p[:,1]=x**2
#p[:,2]=1



#below is what will be inside subroutine


#!! set up the Hessian matrix

def hesfit(x,y,sig,p):
 
 
    sig2=sig*sig
    n_p = p[0,:].shape[0]
 
#calculate the hessian matrix    
    for ip2 in range(n_p):    
        for ip1 in range(n_p):
            hes[ip1,ip2]=np.sum(p[:,ip1]*p[:,ip2]/sig2)

# calculate the covariance matrix    
    cov = np.linalg.inv(hes)
    
    
    
#!!! set up the c matrix in:  a * hes = c (where a is a(NP), hes is the hessian matrix and c = sum (y_i p_i / sig^2_i) see ada lecture 9)
    c=np.zeros(n_p)
    pout=np.zeros(n_p)
    sigpout=np.zeros(n_p)
    
    for ip in range(n_p):
        c[ip]=np.sum(y*p[:,ip]/sig2)
    
    #!stop
    #!! find the solution 
    for ip0 in range(n_p):
        pout[ip0]=np.sum(cov[:,ip0]*c)
        sigpout[ip0]=np.sqrt(cov[ip0,ip0])
    
    #!!
    return(pout,sigpout)




# tested and works 7/10/14 
# general mcmc code python
# IP data: x,y,sigy, func_mod: the model function, nparm: the number of parameters, pstep: best guess of teh step sizes, p: best starting guesses of the parameters, psafe: 0 if step uniformly, 1 if step in log

import numpy as np
import os
import pylab as plt




def mymcmc(x,y,ysig,func_mod,nparm, p, pstep, psafe, p_primean, p_prisig, positivity):
 
 nits = 5000
 checknum = 10
 checkfrac = 0.25
 ploton = 0

 pold = np.zeros(nparm)
 pAT = np.zeros(nparm)
 cisqsave=np.zeros(0)
 parmsave=np.zeros((0,nparm))
 

 checkarange = np.arange(nparm)
 
#optional for plotting purposes (the ion allows the plot to be updated live)
 if (ploton == 1):
  plt.ion()
  fig = plt.figure()
  ax = fig.add_subplot(111)
  ax.plot(x,y)
  yplot = y*0.0
  line1, = ax.plot(x,yplot)
# 
 
# for the bof 
 #idx_pri_log = np.where((p_primean > 0) and ( psafe == 1) )
 #we cant do the above as np.where doesn't work for two arrays
 #instead do it like this
 a = np.logical_and((p_primean > 0), (psafe == 1))
 idx_pri_log=np.where(a == 1)[0]
 
 a = np.logical_and((p_primean > 0), (psafe == 0))
 idx_pri_lin = np.where(a == 1)[0]
 
 
 #cant_start_safe_parms_at_0
 idx_safe_check = np.where(psafe == 1)[0]
 if ( np.any(p[idx_safe_check] <= 0) == 1):
  print 'You cannot make starting values == 0 for parameters stepped in log!'
  raw_input()

 
 
#start the iteration loop 
 
 for iteration in range(nits):


# change step sizes
# keep step sizes reasonable
  reset_check = np.mod(iteration,checknum)
  if (reset_check == 0):
   idxup = np.where(pAT/checknum > checkfrac)[0] #these steps are being doubled
   idxdown = np.where(checkarange != idxup)[0] #these steps are bing halved
   pstep[idxup] = 2*pstep[idxup]
   pstep[idxdown] = 0.5*pstep[idxdown]
   pAT[:] = 0
# finished changing step sizes

#step in the parameter or log of the parameter
  for ip in range(nparm):
   pip = p[ip]
   pipstep = pstep[ip]
   pold[ip] = pip

# step in the parameter (or the of the parameter if psafe =1)  
   a = np.random.randn(1)*pipstep 
   if (psafe[ip] ==1):
    logp = np.log10(pip)
    pip = 10**(logp + a)
   else:
    pip = pip + a
  
   #if (ip == 1):
    #print pip,a,logp, iteration, 'psafe bug'
    #raw_input()
    
  #print 'parm test', pip, a, p[ip]
   p[ip] = pip
  
  
# evaluate the model with the new parameters. p is the 1d array of parameters
   ymod = func_mod(x,p)
   #print ymod
   if (positivity == 1):
    ymod = np.exp(ymod) - 1
   #print ymod
   #print ''
# calculate the cisq of the new model wrt data
   a = y - ymod
   cisq = np.sum( a*a/(ysig*ysig) )

# add on any priors   
   a = (p[idx_pri_lin]- p_primean[idx_pri_lin])/p_prisig[idx_pri_lin]
   bof = np.sum(a*a)
   
   #print a
   #print bof
   #print 'bof bug'
   #raw_input()
   
   a = (np.log10(p[idx_pri_log]) - p_primean[idx_pri_log])/p_prisig[idx_pri_log]
   bof = bof +np.sum(a*a) 
   
   #print p[idx_pri_log], p_primean[idx_pri_log], p_prisig[idx_pri_log]
   #print a
   #print bof
   #print 'bof bug 2'
   #raw_input()
    
   bof = bof + cisq
   
   
# decide to oaccept
   cisqnew = cisq
   bofnew = bof
   if ( (iteration == 0) and (ip == 0) ):
    cisqold = cisq
    bofold = bof
# accepts 
  #print cisqnew,cisqold,'cisqtest' 
  #raw_input()
   if ( (bofnew < bofold) or (iteration == 1) or (np.exp(-0.5*(bofnew-bofold)) > np.random.rand(1)) ):
    pAT[ip] = pAT[ip]+1
    cisqold = cisqnew
    bofold  = bofnew
# reject   
   else:
    p[ip] = pold[ip]

  cisqsave = np.append(cisqsave,cisqold) #save the bof
  parmsave = np.vstack( [parmsave, [p[:]] ] )

 
  print iteration
  print p
  print pstep
  print bofold
  #raw_input()
# optional bit (plot the result of each iteration for comparison
  if (ploton == 1):
   yplot = func_mod(x,p)
   if (positivity == 1):
    yplot=np.exp(yplot) - 1
   #print 'poditive'
   #xmodtest = np.arange(-5,5,0.1)
   #ymodtest=np.exp(func_mod(xmodtest
   line1.set_ydata(yplot)
   fig.canvas.draw()
  #print yplot[:3]
  #print y[:3]
  #print x[:3]
  #raw_input()
  
 return(cisqsave,parmsave)
 


#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  
 
#test inputs (these will be turned into inputs for a definition
#x = np.arange(10)
#y = 2*x + 3
#nx = x.shape[0]
#ysig = np.ones(nx)*x.mean()/10
#
#
#
#
#def f(xin,a,b):
# return(a*xin+b)
#
#nparm = 2
#pstep = np.ones(nparm)
#p=np.zeros(nparm)
#psafe = np.zeros(nparm)
#
#op = mymcmc(x,y,ysig,f,nparm, p, pstep, psafe)
#